/**
 * COMP 401 Fall 2017 -- Recitation 13 October 2017
 * 
 * Copyright (C) 2017, Alex Blate. All Rights Reserved.
 */

package recitations.recitation8;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Objects;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;

import bus.uigen.CompleteOEFrame;
import bus.uigen.ObjectEditor;


public class AppDemo1 implements PropertyChangeListener {

	private static final String	LINE1_CONTROL_TITLE	= "Line 1 Controls";
	public static final int		MAX_WIDTH			= 100;
	public static final int		MOVE_INCR			= 2;

	public static final int		UPDATE_PERIOD_MS	= 1000;
	public static final int		EDITOR_HEIGHT		= 700;
	public static final int		EDITOR_WIDTH		= 700;

	protected CompleteOEFrame	editor				= null;

	public AppDemo1() {
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		//POI #4
		//		System.out.println("Received event: " + evt.toString());
		if (null != editor) {
			editor.refresh();			
		}
		
	}

	public void start() {

		//POI #1
		StrokedLine line1 = new AStrokedLine(10, 10, 100, 100, 2F);
		//		StrokedLine line1 = new AnObservableStrokedLine(10, 10, 100, 100, 2F);

		//POI #2
		StrokedLine line2 = new AnAutoScaledStrokedLine(30, 40, 30, 40, 10);
		//		StrokedLine line2 = new AnObservableStrokedLine(new AnAutoScaledStrokedLine(30, 40, 30, 40, 10));

		@SuppressWarnings("unused")
		ConsoleListener consoleListener = new ConsoleListener();

		StrokedLine[] lines = { line1, line2 };

		//POI #5B
		for (StrokedLine line : lines) {
			if (line instanceof Listenable) {
				//POI #3
				((Listenable) line).addPropertyChangeListener(this);
				//POI #8
				//((Listenable) line).addPropertyChangeListener(consoleListener);
			}
		}

		editor = ObjectEditor.edit(lines);
		editor.setSize(EDITOR_WIDTH, EDITOR_HEIGHT);
		

		lineAnimatorThread(line1, line2).start();		

		createLineControl(line1, LINE1_CONTROL_TITLE);

	}

	/**
	 * Make a little window with movement buttons for the given Line
	 * @param line
	 * @param windowTitle
	 */
	public static JFrame createLineControl(Line line, String title) {
		JFrame frame = new JFrame(title);
		BoxLayout boxLayout = new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS);

		frame.setSize(100, 100);
		frame.setLayout(boxLayout);
		
		JButton leftButton = new JButton("Left");
		leftButton.addActionListener(new LineMoverAction(line, MoveDirection.LEFT, MOVE_INCR));
		
		JButton rightButton = new JButton("Right");
		rightButton.addActionListener(new LineMoverAction(line, MoveDirection.RIGHT, MOVE_INCR));

		frame.add(leftButton);
		frame.add(rightButton);
		frame.setVisible(true);
		
		return frame;
	}

	/**
	 * Animate lines asynchronously to the program
	 * @param line1
	 * @param line2
	 * @return new Thread to perform animation
	 */
	public Thread lineAnimatorThread(StrokedLine line1, StrokedLine line2) {
		Objects.requireNonNull(line1);
		Objects.requireNonNull(line2);
		
		/**
		 * This is an example of instantiating an anonymous interface;
		 * this could also be done by writing a new class that implements Runnable
		 */
		Thread thr = new Thread(new Runnable() {

			@Override
			public void run() {
				while (true) {
					try {
						Thread.sleep(UPDATE_PERIOD_MS);
					} catch (InterruptedException ex) {
						//ignore
					}

					double currentWidth = line1.getStrokeWidth();
					if (currentWidth < MAX_WIDTH) {
						line1.changeStrokeWidth(currentWidth + 1);
					} else {
						line1.changeStrokeWidth(1);
					}

					//					System.out.println("New width: " + line1.getStrokeWidth());

					line2.setHeight(line2.getWidth() + 1);
					line2.setWidth(line2.getHeight() + 1);
					//POI #9
					line2.setY((line2.getY() + 3) % EDITOR_HEIGHT);
				}//while true

			}//run
		}); //new thread

		return thr;
	}//lineAnimatorThread
	
	
}//class
